/*
 * ComplexType.java
 * Copyright (C) 2007 Lars Frantzen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 *
 * Created on 29. Dezember 2006, 16:39
 *
 */

package info.frantzen.testing.ssmsimulator.types;

import java.util.HashSet;
import javax.xml.namespace.QName;

/**
 * A complex type is similar to an XML-Schema complex type, or what is called a
 * <I>record</I> in programming languages. A complex type is a list of (name,{@link Type type}) pairs.<br>
 * <br>
 * Assuming you have already {@link Type type} objects called <CODE>thePosDouble</CODE>, <CODE>product</CODE>,
 * and <CODE>thePosInt</CODE>. You create a complex type for instance like this:<br>
 * <CODE>
 *        Object[][] qu = new Object[5][2];<br>
 *        qu[0][0] = "price";<br>
 *        qu[0][1] = thePosDouble;<br>
 *        qu[1][0] = "product";<br>
 *        qu[1][1] = product;<br>
 *        qu[2][0] = "quantity";<br>
 *        qu[2][1] = thePosInt;<br>
 *        qu[3][0] = "refNumber";<br>
 *        qu[3][1] = thePosInt;<br>
 *        qu[4][0] = "sum";<br>
 *        qu[4][1] = thePosDouble;<br>
 *        ComplexType quote = new ComplexType("quote", qu);<br>
 * </CODE>
 * @author Lars Frantzen
 * @version 0.4
 */
public class ComplexType extends Type {
    
    private Object[][] fields;
    
    /**
     * Creates a new instance of ComplexType.
     * @param name the name of the complex type
     * @param fields a two-dimensional array of (name,{@link Type type}) pairs
     */
    public ComplexType(QName name, Object[][] fields) {
        super(name);
        this.fields = fields;
    }
    
    /**
     * Creates a new instance of ComplexType without setting the fields.
     * @param name the name of the complex type
     */
    public ComplexType(QName name) {
        super(name);
    }
    
    /**
     * Sets the fields of the complex type.
     * @param fields a two-dimensional array of (name,{@link Type type}) pairs
     */
    public void setFields(Object[][] fields) {
        this.fields = fields;
    }
    
    // Default implementations
    /**
     * Yields a random {@link ComplexTypeInstance complex type instance} by delegating
     * the generation of random field values to the single types of the complex type.
     * @return a complex type instance with random values
     */
    public ComplexTypeInstance generateRandomInstance() {
        Object[] fieldvalues = new Object[fields.length];
        
        for (int i=0; i<fields.length; i++)
            fieldvalues[i] = ((Type) fields[i][1]).generateRandomInstance();
        
        return new ComplexTypeInstance(fieldvalues);
    }
    
    /**
     * Yields the next random {@link ComplexTypeInstance complex type instance} by delegating
     * the generation of the next random field values to the single types of the complex type.
     * @return the next complex type instance with random values
     */
    public ComplexTypeInstance generateNextBoundaryInstance() {
        Object[] fieldvalues = new Object[fields.length];
        
        for (int i=0; i<fields.length; i++)
            fieldvalues[i] = ((Type) fields[i][1]).generateNextBoundaryInstance();
        
        return new ComplexTypeInstance(fieldvalues);
    }
    
    /**
     * Here we return the sum of the boundary instances of the fields.
     * @return the sum of the boundary instances of the fields
     */
    public int numberOfBoundaryInstances() {
        int result=0;
        for (int i=0; i<fields.length; i++)
            result += ((Type) fields[i][1]).numberOfBoundaryInstances();
        return result;
    }
    
    /**
     * Getter for the fields of the complex type.
     * @return the fields of the complex type
     */
    public Object[][] getFields() {
        return fields;
    }
    
    
    /**
     * Yields the initial {@link ComplexTypeInstance complex type instance} by delegating
     * the generation of initial field values to the single types of the complex type.
     * This leads to a stack overflow in case of recursive self references!
     * @return a complex type instance with initial values
     */
    public ComplexTypeInstance generateInitialInstance() {
        Object[] fieldvalues = new Object[fields.length];
        
        for (int i=0; i<fields.length; i++)
            fieldvalues[i] = ((Type) fields[i][1]).generateInitialInstance();
        
        return new ComplexTypeInstance(fieldvalues);
    }
    
    /**
     * Checks the consistency of the complex type, i.e. it is checked if a name of
     * a field is the empty string, or if a type is null.
     * @return true, if the consisitency is true, false otherwise
     */
    public boolean checkConsistency() {
        HashSet<String> names = new HashSet<String>();
        for (int i=0; i<fields.length; i++) {
            String name = (String) fields[i][0];
            if ((name == "") || ((Type) fields[i][1] == null))
                return false;
            if (names.contains(name))
                return false;
            names.add(name);
        }
        return true;
    }
    
    /**
     * A string representation of the complex type.
     * @return a string representation of the complex type
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append(getName() + '[');
        for (int i=0; i<fields.length; i++) {
            sb.append((String) fields[i][0]);
            sb.append(':');
            sb.append((Type) fields[i][1]);
            if (!(i == fields.length-1))
                sb.append(',');
        }
        
        sb.append(']');
        return sb.toString();
    }

    public String getTypeIdentifier() {
        return "ComplexType";
    }
    
}
