/*
 * Enumeration.java
 * Copyright (C) 2007 Lars Frantzen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 *
 * Created on 29. Dezember 2006, 17:12
 *
 */

package info.frantzen.testing.ssmsimulator.types;

import javax.xml.namespace.QName;

/**
 * An enumeration corresponds to a list of names.
 * @author Lars Frantzen
 * @version 0.3
 */
public class Enumeration extends Type {
    
    private String[] names;
    int nextInstance = 0;
    
    /**
     * Creates a new instance of Enumeration.
     * @param name the name of the enumeration
     * @param names the list of names of this enumeration
     */
    public Enumeration(QName name, String[] names) {
        super(name);
        this.names = names;
    }
    
    // Default implementations
    /**
     * Yields a random {@link EnumerationInstance enumeration instance} by choosing a
     * random element of this enumeration.
     * @return a random enumeration instance
     */
    public EnumerationInstance generateRandomInstance() {
        return new EnumerationInstance(this, names[random.nextInt(names.length)]);
    }
    
    /**
     * Yields the next random {@link EnumerationInstance enumeration instance} by choosing always the
     * next element of this enumeration (modulo its length). Initially the first element is returned.
     * @return the next random enumeration instance
     */
    public EnumerationInstance generateNextBoundaryInstance() {
        if (nextInstance == names.length)
            nextInstance = 0;
        return new EnumerationInstance(this, names[nextInstance++]);
    }
    
    /**
     * Returns the number of boundary instances. For an enumeration this corresponds to the
     * number of elements in the enumeration.
     * @return the number of boundary instances
     */
    public int numberOfBoundaryInstances() {
        return names.length;
    }
    
    /**
     * Getter for the names of this enumeration.
     * @return the names of this enumeration
     */
    public String[] getNames() {
        return names;
    }
    
    /**
     * Yields the initial {@link EnumerationInstance enumeration instance}.
     * We return here the first name of the enumeration.
     * @return the initial enumeration instance
     */
    public EnumerationInstance generateInitialInstance() {
        return new EnumerationInstance(this, names[0]);
    }
    
    /**
     * Checks the consistency of the enumeration, i.e. it is checked if the name of the
     * enumeration is not the empty string, and if the array of elements has at least one element
     * and is not null.
     * @return true, if the consisitency is true, false otherwise
     */
    public boolean checkConsistency() {
        return ((super.getName() != null) && (names != null) && (names.length != 0));
    }
    
    public int instanceAsNumber(EnumerationInstance instance) {
        for (int i = 0; i < names.length; i++)
            if (names[i].equals((String) instance.getValue()))
                return i;
        
        return -1;
    }
    
    public EnumerationInstance instanceFromNumber(int number) {
        return new EnumerationInstance(this, names[number]);
    }
    
    /**
     * A string representation of the enumeration.
     * @return a string representation of the enumeration
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append(getName());
        
        sb.append('{');
        for (int i = 0; i < names.length; i++) {
            sb.append(names[i]);
            if (!(i == names.length-1))
                sb.append(',');
            
        }
        sb.append('}');
        
        return sb.toString();
    }

    public String getTypeIdentifier() {
        return "Enumeration";
    }
    
}
